# Generated by Django 5.1.7 on 2025-06-09 12:34

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('ixp_tracker', '0010_statspercountry_ixp_count'),
    ]

    operations = [
        migrations.AddField(
            model_name='statsperixp',
            name='rs_peering_rate',
            field=models.FloatField(default=None),
        ),
    ]
