# Generated by Django 5.0.8 on 2025-04-24 09:27

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('ixp_tracker', '0009_alter_ixpmembershiprecord_member'),
    ]

    operations = [
        migrations.AddField(
            model_name='statspercountry',
            name='ixp_count',
            field=models.IntegerField(default=0),
            preserve_default=False,
        ),
    ]
